%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%             
%      Promoting Partial Reconstruction for The Morphological Analysis 
%              of Very High Resolution Remote Sensing Images
%                            
% 
%           
%       Copyright notes
%       Author: Wenzhi Liao, IPI, Telin, Ghent University, Belgium
%       Date: 19/07/2016
%
%
%
%       Please cite the following references:
%
%       [1] W. Liao, J. Chanussot, M. Dalla Mura, X. Huang, R. Bellens,
%           S. Gautama, W, Philips, "Taking Optimal Advantage of Fine Spatial 
%           Resolution: Promoting partial image reconstruction for the 
%           morphological analysis of very-high-resolution images",  IEEE 
%           Geoscience and Remote Sensing Magazine, vol. 5, no. 2, pp. 8-28, 2017.
%
%       [2] W. Liao, M. Dalla Mura, J. Chanussot, R. Bellens, W. Philips, 
%           "Morphological Attribute Profiles With Partial Reconstruction", 
%           IEEE Transactions on Geoscience and Remote Sensing, vol. 54, 
%           no. 3, pp. 1738-1756, 2016.
%
%       [3] R. Bellens, J. Chan, L. Martinez-Fonte, S. Gautama, F. Canters, 
%           W. Philips, "Improved classification of VHR images of urban areas 
%           using directional morphological profiles", IEEE Transactions on
%           Geoscience and Remote Sensing, vol. 46, no. 10, pp. 2803-2813, 2008.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc; clear all
close all
%% add path
p = fileparts(mfilename('fullpath'));
addpath(fullfile(p,'Images'));
addpath(fullfile(p,'Attribute_profiles'));
%% load image
load HS_Image.mat
PCs=double(PCs_Pavia);% the first 3 principal components of Pavia University Hyperspectral image
clear PCs_Pavia
[rows, cols, dims]=size(PCs);

%% Compute the morphological profiles with no reconstruction, with reconstruction [3], and with partial reconstruction [2]
type_se='disk'; %'square';%'line';% octagon, dodecagon, hexadecagon,  bresenhamline
num_scales=6; %number of scales for morphological profiles

%% Morphological profiles with no reconstruction based on PCA
MPN=[];
for i=1:dims
   MPNeachpca=getmorphprofiles(PCs(:,:,i),'euclidean',type_se,num_scales);%
   MPN=cat(3,MPN,MPNeachpca);        
end
MPN=cat(3,MPN,PCs);
%% Morphological profiles with reconstruction based on PCA
MPR=[];
for i=1:dims
   MPReachpca=getmorphprofiles(PCs(:,:,i),'geodesic',type_se,num_scales);
   MPR=cat(3,MPR,MPReachpca);        
end
MPR=cat(3,MPR,PCs);
%% Morphological profiles with partial reconstruction based on PCA
MPPR=[];
for i=1:dims
   MPPReachpca=getmorphprofiles(PCs(:,:,i),'partialreconstruction',type_se,num_scales);
   MPPR=cat(3,MPPR,MPPReachpca);        
end
MPPR=cat(3,MPPR,PCs);

figure;
subplot(3,3,1);imshow(MPN(:,:,2),[])
title('SE=2')
subplot(3,3,2);imshow(MPN(:,:,4),[])
title('SE=4')
subplot(3,3,3);imshow(MPN(:,:,6),[])
title('SE=6')
subplot(3,3,4);imshow(MPR(:,:,2),[])
subplot(3,3,5);imshow(MPR(:,:,4),[])
title('Full reconstruction')
subplot(3,3,6);imshow(MPR(:,:,6),[])
subplot(3,3,7);imshow(MPPR(:,:,2),[])
subplot(3,3,8);imshow(MPPR(:,:,4),[])
title('Partial reconstruction')
subplot(3,3,9);imshow(MPPR(:,:,6),[])

%% Compute the attribute profiles with conventional attribute filters [4] and attribute filters with partial reconstruction [1]
Re_Scale=10; %convert the principal components into integer, larger values require more computational time
for i=1:dims
    Temp=PCs(:,:,i);
    Temp=Re_Scale*(Temp-min(Temp(:)))/(max(Temp(:))-min(Temp(:)));
    PCs(:,:,i)=double(uint16(Temp));
end

Area=[500, 1000, 5000];
%Std=[2, 4, 6];
%MI=[0.1, 0.3, 0.5];
APa=[];APPRa=[];
%APs=[];APPRs=[];
%APi=[];APPRi=[];

% Attribute filters with partial reconstruction
tStart_PR=tic;
for j=1:1%dims
    for i=1:length(Area)
        APPRa_OP=attrib_filter_PR(PCs(:,:,j), 'area', Area(i), 'open');
        APPRa_CL=attrib_filter_PR(PCs(:,:,j), 'area', Area(i), 'close');
        APPRa=cat(3,APPRa,APPRa_OP,APPRa_CL);
%        APPRs_OP=attrib_filter_PR(PCs(:,:,j), 'sd', Std(i), 'open');
%        APPRs_CL=attrib_filter_PR(PCs(:,:,j), 'sd', Std(i), 'close');
%        APPRs=cat(3,APPRs,APPRs_OP,APPRs_CL);
%        APPRi_OP=attrib_filter_PR(PCs(:,:,j), 'mi', MI(i), 'open');
%        APPRi_CL=attrib_filter_PR(PCs(:,:,j), 'mi', MI(i), 'close');
%        APPRi=cat(3,APPRi,APPRi_OP,APPRi_CL);
    end
end
tElapsed_PR=toc(tStart_PR)


% Conventional attribute filters 
tStart_Org=tic;
for j=1:1%dims
    for i=1:length(Area)
        APa_OP=attrib_filter(PCs(:,:,j), 'area', Area(i), 'open');
        APa_CL=attrib_filter(PCs(:,:,j), 'area', Area(i), 'close');
        APa=cat(3,APa,APa_OP,APa_CL);
%        APs_OP=attrib_filter(PCs(:,:,j), 'sd', Std(i), 'open');
%        APs_CL=attrib_filter(PCs(:,:,j), 'sd', Std(i), 'close');
%        APs=cat(3,APs,APs_OP,APs_CL);
%        APi_OP=attrib_filter(PCs(:,:,j), 'mi', MI(i), 'open');
%        APi_CL=attrib_filter(PCs(:,:,j), 'mi', MI(i), 'close');
%        APi=cat(3,APi,APi_OP,APi_CL);
    end
end
tElapsed_Org=toc(tStart_Org)



figure;
subplot(2,3,1);imshow(APa(:,:,1),[])
title('Area=500')
subplot(2,3,2);imshow(APa(:,:,3),[])
title('Area=1000')
subplot(2,3,3);imshow(APa(:,:,5),[])
title('Area=5000')
subplot(2,3,4);imshow(APPRa(:,:,1),[])
subplot(2,3,5);imshow(APPRa(:,:,3),[])
title('Attribute profiles with partial reconstruction')
subplot(2,3,6);imshow(APPRa(:,:,5),[])


