%       Please cite the following references:
%
%       [1] W. Liao, J. Chanussot, M. Dalla Mura, X. Huang, R. Bellens,
%           S. Gautama, W, Philips, "Taking Optimal Advantage of Fine Spatial 
%           Resolution: Promoting partial image reconstruction for the 
%           morphological analysis of very-high-resolution images",  IEEE 
%           Geoscience and Remote Sensing Magazine, vol. 5, no. 2, pp. 8-28, 2017.
%
%       [2] W. Liao, M. Dalla Mura, J. Chanussot, R. Bellens, W. Philips, 
%           "Morphological Attribute Profiles With Partial Reconstruction", 
%           IEEE Transactions on Geoscience and Remote Sensing, vol. 54, 
%           no. 3, pp. 1738-1756, 2016.
%
%       [3] R. Bellens, J. Chan, L. Martinez-Fonte, S. Gautama, F. Canters, 
%           W. Philips, "Improved classification of VHR images of urban areas 
%           using directional morphological profiles", IEEE Transactions on
%           Geoscience and Remote Sensing, vol. 46, no. 10, pp. 2803-2813, 2008.
%
%
%
%       Copyright notes
%       Author: Rik Bellens and Wenzhi Liao, IPI, Telin, Ghent University, Belgium
%       Date: 19/07/2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



classdef vhgwstrel < basestrel
    
    properties (Access = protected)
        length;
    end
    
    methods (Access = public)
        
        function se = reflect(se)
        end

        function out = imopen(im,se,varargin)
            out = morphop('imopen',im,se,varargin{:});
        end
        
        function out = imclose(im,se,varargin)
            out = morphop('imclose',im,se,varargin{:});
        end
    end

    methods (Access = protected)
        function obj = vhgwstrel(hlength,matlabse)
            obj@basestrel(matlabse);
            obj.length = hlength;
        end
        
        function out = rotateimage(im,se,b)
            error('abstract');
        end
         function out = unrotateimage(im,se,h,w)
            error('abstract');
        end
   
        function out = morphop(op,im,se,shape)
            
            switch op
                case {'imdilate','imclose'}
                    b = -inf;
                case {'imerode','imopen'}
                    b = inf;
                otherwise 
                    
                    error('...');
            end
            [h,w] = size(im);
            
            im = rotateimage(im,se,b);
            
            vertse = strel('periodicline',se.length,[1 0]);
            out = feval(op,im,vertse);
            
            
            out = unrotateimage(out,se,h,w);
            s = getboundarysize(se);
            if nargin<4||strcmp(shape,'same')
                out = out( s(1) + (1:h), s(2) + (1:w) );
            end
        end
    end
end
