%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     Output:
%             morph_profiles: morphlogical profiles containing openings and
%             closings;
%    
%     Input:  
%             input:          input image
%             type_g:         the type of granulometry
%             type_se:        the type of the structure elements
%             num_scale:      the scales of openings and closings
%
%       Copyright notes
%       Author: Wenzhi Liao, IPI, Telin, Ghent University, Belgium
%       Date: 19/07/2016
%
%
%       Please cite the following references:
%
%       [1] W. Liao, J. Chanussot, M. Dalla Mura, X. Huang, R. Bellens,
%           S. Gautama, W, Philips, "Taking Optimal Advantage of Fine Spatial 
%           Resolution: Promoting partial image reconstruction for the 
%           morphological analysis of very-high-resolution images",  IEEE 
%           Geoscience and Remote Sensing Magazine, vol. 5, no. 2, pp. 8-28, 2017.
%
%       [2] W. Liao, M. Dalla Mura, J. Chanussot, R. Bellens, W. Philips, 
%           "Morphological Attribute Profiles With Partial Reconstruction", 
%           IEEE Transactions on Geoscience and Remote Sensing, vol. 54, 
%           no. 3, pp. 1738-1756, 2016.
%
%       [3] R. Bellens, J. Chan, L. Martinez-Fonte, S. Gautama, F. Canters, 
%           W. Philips, "Improved classification of VHR images of urban areas 
%           using directional morphological profiles", IEEE Transactions on
%           Geoscience and Remote Sensing, vol. 46, no. 10, pp. 2803-2813, 2008.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function morph_profiles=getmorphprofiles(input,type_g,type_se,num_scales)

p = fileparts(mfilename('fullpath'));
addpath(fullfile(p,'fastmorphology'));
addpath(fullfile(p,'granulometry'));

train_image=input;
scales=num_scales;
operator=type_g;


g=granulometry(operator,type_se);

%create openings with scales
open_profiles = GenerateFirstNScales(g,train_image,scales);
%create closings with scales
close_profiles = GenerateFirstNScales(g,imcomplement(train_image),scales);

for i=1:scales
  close_profiles{i}=imcomplement(close_profiles{i});
end

morph_profiles=[open_profiles, close_profiles];

morph_profiles=double(cat(3, morph_profiles{:}));
